@@ -99,10 +99,10 @@ struct GsdKeyboardManagerPrivate
         GSettings *input_sources_settings;
         GSettings *interface_settings;
         GnomeXkbInfo *xkb_info;
+        GDBusProxy *localed;
 #ifdef HAVE_IBUS
         IBusBus   *ibus;
         GHashTable *ibus_engines;
-        GHashTable *ibus_xkb_engines;
         GCancellable *ibus_cancellable;
 #endif
         gint       xkb_event_base;
@@ -123,7 +123,7 @@ static gboolean apply_input_sources_settings     (GSettings               *setti
                                                   gint                     n_keys,
                                                   GsdKeyboardManager      *manager);
 static void     set_gtk_im_module                (GsdKeyboardManager      *manager,
-                                                  const gchar             *new_module);
+                                                  GVariant                *sources);
 
 G_DEFINE_TYPE (GsdKeyboardManager, gsd_keyboard_manager, G_TYPE_OBJECT)
 
@@ -172,49 +172,9 @@ clear_ibus (GsdKeyboardManager *manager)
         g_cancellable_cancel (priv->ibus_cancellable);
         g_clear_object (&priv->ibus_cancellable);
         g_clear_pointer (&priv->ibus_engines, g_hash_table_destroy);
-        g_clear_pointer (&priv->ibus_xkb_engines, g_hash_table_destroy);
         g_clear_object (&priv->ibus);
 }
 
-static gchar *
-make_xkb_source_id (const gchar *engine_id)
-{
-        gchar *id;
-        gchar *p;
-        gint n_colons = 0;
-
-        /* engine_id is like "xkb:layout:variant:lang" where
-         * 'variant' and 'lang' might be empty */
-
-        engine_id += 4;
-
-        for (p = (gchar *)engine_id; *p; ++p)
-                if (*p == ':')
-                        if (++n_colons == 2)
-                                break;
-        if (!*p)
-                return NULL;
-
-        id = g_strndup (engine_id, p - engine_id + 1);
-
-        id[p - engine_id] = '\0';
-
-        /* id is "layout:variant" where 'variant' might be empty */
-
-        for (p = id; *p; ++p)
-                if (*p == ':') {
-                        if (*(p + 1) == '\0')
-                                *p = '\0';
-                        else
-                                *p = '+';
-                        break;
-                }
-
-        /* id is "layout+variant" or "layout" */
-
-        return id;
-}
-
 static void
 fetch_ibus_engines_result (GObject            *object,
                            GAsyncResult       *result,
@@ -243,22 +203,12 @@ fetch_ibus_engines_result (GObject            *object,
 
         /* Maps IBus engine ids to engine description objects */
         priv->ibus_engines = g_hash_table_new_full (g_str_hash, g_str_equal, NULL, g_object_unref);
-        /* Maps XKB source id strings to engine description objects */
-        priv->ibus_xkb_engines = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
 
         for (l = list; l; l = l->next) {
                 IBusEngineDesc *engine = l->data;
                 const gchar *engine_id = ibus_engine_desc_get_name (engine);
 
                 g_hash_table_replace (priv->ibus_engines, (gpointer)engine_id, engine);
-
-                if (strncmp ("xkb:", engine_id, 4) == 0) {
-                        gchar *xkb_source_id = make_xkb_source_id (engine_id);
-                        if (xkb_source_id)
-                                g_hash_table_replace (priv->ibus_xkb_engines,
-                                                      xkb_source_id,
-                                                      engine);
-                }
         }
         g_list_free (list);
 
@@ -284,23 +234,8 @@ fetch_ibus_engines (GsdKeyboardManager *manager)
 }
 
 static void
-maybe_start_ibus (GsdKeyboardManager *manager,
-                  GVariant           *sources)
+maybe_start_ibus (GsdKeyboardManager *manager)
 {
-        gboolean need_ibus = FALSE;
-        GVariantIter iter;
-        const gchar *type;
-
-        g_variant_iter_init (&iter, sources);
-        while (g_variant_iter_next (&iter, "(&s&s)", &type, NULL))
-                if (g_str_equal (type, INPUT_SOURCE_TYPE_IBUS)) {
-                        need_ibus = TRUE;
-                        break;
-                }
-
-        if (!need_ibus)
-                return;
-
         if (!manager->priv->ibus) {
                 ibus_init ();
                 manager->priv->ibus = ibus_bus_new_async ();
@@ -363,22 +298,64 @@ set_ibus_engine (GsdKeyboardManager *manager,
 }
 
 static void
-set_ibus_xkb_engine (GsdKeyboardManager *manager,
-                     const gchar        *xkb_id)
+set_ibus_xkb_engine (GsdKeyboardManager *manager)
 {
         IBusEngineDesc *engine;
         GsdKeyboardManagerPrivate *priv = manager->priv;
 
-        if (!priv->ibus_xkb_engines)
+        if (!priv->ibus_engines)
                 return;
 
-        engine = g_hash_table_lookup (priv->ibus_xkb_engines, xkb_id);
+        /* All the "xkb:..." IBus engines simply "echo" back symbols,
+           despite their naming implying differently, so we always set
+           one in order for XIM applications to work given that we set
+           XMODIFIERS=@im=ibus in the first place so that they can
+           work without restarting when/if the user adds an IBus
+           input source. */
+        engine = g_hash_table_lookup (priv->ibus_engines, "xkb:us::eng");
         if (!engine)
                 return;
 
         set_ibus_engine (manager, ibus_engine_desc_get_name (engine));
 }
 
+static gboolean
+need_ibus (GVariant *sources)
+{
+        GVariantIter iter;
+        const gchar *type;
+
+        g_variant_iter_init (&iter, sources);
+        while (g_variant_iter_next (&iter, "(&s&s)", &type, NULL))
+                if (g_str_equal (type, INPUT_SOURCE_TYPE_IBUS))
+                        return TRUE;
+
+        return FALSE;
+}
+
+
+static void
+set_gtk_im_module (GsdKeyboardManager *manager,
+                   GVariant           *sources)
+{
+        GsdKeyboardManagerPrivate *priv = manager->priv;
+        const gchar *new_module;
+        gchar *current_module;
+
+        if (!sources || need_ibus (sources))
+                new_module = GTK_IM_MODULE_IBUS;
+        else
+                new_module = GTK_IM_MODULE_SIMPLE;
+
+        current_module = g_settings_get_string (priv->interface_settings,
+                                                KEY_GTK_IM_MODULE);
+        if (!g_str_equal (current_module, new_module))
+                g_settings_set_string (priv->interface_settings,
+                                       KEY_GTK_IM_MODULE,
+                                       new_module);
+        g_free (current_module);
+}
+
 /* XXX: See upstream bug:
  * https://codereview.appspot.com/6586075/ */
 static gchar *
@@ -963,22 +940,6 @@ apply_xkb_settings (GsdKeyboardManager *manager,
         g_free (rules_file_path);
 }
 
-static void
-set_gtk_im_module (GsdKeyboardManager *manager,
-                   const gchar        *new_module)
-{
-        GsdKeyboardManagerPrivate *priv = manager->priv;
-        gchar *current_module;
-
-        current_module = g_settings_get_string (priv->interface_settings,
-                                                KEY_GTK_IM_MODULE);
-        if (!g_str_equal (current_module, new_module))
-                g_settings_set_string (priv->interface_settings,
-                                       KEY_GTK_IM_MODULE,
-                                       new_module);
-        g_free (current_module);
-}
-
 static gboolean
 apply_input_sources_settings (GSettings          *settings,
                               gpointer            keys,
@@ -1010,7 +971,7 @@ apply_input_sources_settings (GSettings          *settings,
         }
 
 #ifdef HAVE_IBUS
-        maybe_start_ibus (manager, sources);
+        maybe_start_ibus (manager);
 #endif
 
         g_variant_get_child (sources, current, "(&s&s)", &type, &id);
@@ -1026,9 +987,9 @@ apply_input_sources_settings (GSettings          *settings,
                         g_warning ("Couldn't find XKB input source '%s'", id);
                         goto exit;
                 }
-                set_gtk_im_module (manager, GTK_IM_MODULE_SIMPLE);
 #ifdef HAVE_IBUS
-                set_ibus_xkb_engine (manager, id);
+                set_gtk_im_module (manager, sources);
+                set_ibus_xkb_engine (manager);
 #endif
         } else if (g_str_equal (type, INPUT_SOURCE_TYPE_IBUS)) {
 #ifdef HAVE_IBUS
@@ -1053,7 +1014,9 @@ apply_input_sources_settings (GSettings          *settings,
                         goto exit;
                 }
 
-                set_gtk_im_module (manager, GTK_IM_MODULE_IBUS);
+                /* NULL here is a shortcut for "I already know I
+                   need the IBus module". */
+                set_gtk_im_module (manager, NULL);
                 set_ibus_engine (manager, id);
 #else
                 g_warning ("IBus input source type specified but IBus support was not compiled");
@@ -1293,49 +1256,81 @@ set_devicepresence_handler (GsdKeyboardManager *manager)
 }
 
 static void
-create_sources_from_current_xkb_config (GSettings *settings)
+get_sources_from_xkb_config (GsdKeyboardManager *manager)
 {
+        GsdKeyboardManagerPrivate *priv = manager->priv;
         GVariantBuilder builder;
-        XkbRF_VarDefsRec *xkb_var_defs;
-        gchar *tmp;
+        GVariant *v;
+        gint i, n;
         gchar **layouts = NULL;
         gchar **variants = NULL;
-        guint i, n;
 
-        gnome_xkb_info_get_var_defs (&tmp, &xkb_var_defs);
-        g_free (tmp);
-
-        if (xkb_var_defs->layout)
-                layouts = g_strsplit (xkb_var_defs->layout, ",", 0);
-        if (xkb_var_defs->variant)
-                variants = g_strsplit (xkb_var_defs->variant, ",", 0);
-
-        gnome_xkb_info_free_var_defs (xkb_var_defs);
+        v = g_dbus_proxy_get_cached_property (priv->localed, "X11Layout");
+        if (v) {
+                const gchar *s = g_variant_get_string (v, NULL);
+                if (*s)
+                        layouts = g_strsplit (s, ",", -1);
+                g_variant_unref (v);
+        }
 
         if (!layouts)
-                goto out;
+                return;
+
+        v = g_dbus_proxy_get_cached_property (priv->localed, "X11Variant");
+        if (v) {
+                const gchar *s = g_variant_get_string (v, NULL);
+                if (*s)
+                        variants = g_strsplit (s, ",", -1);
+                g_variant_unref (v);
+        }
 
         if (variants && variants[0])
                 n = MIN (g_strv_length (layouts), g_strv_length (variants));
         else
                 n = g_strv_length (layouts);
 
-        g_variant_builder_init (&builder, G_VARIANT_TYPE ("a(ss)"));
-        for (i = 0; i < n && layouts[i][0]; ++i) {
-                if (variants && variants[i] && variants[i][0])
-                        tmp = g_strdup_printf ("%s+%s", layouts[i], variants[i]);
-                else
-                        tmp = g_strdup (layouts[i]);
+        init_builder_with_sources (&builder, priv->input_sources_settings);
 
-                g_variant_builder_add (&builder, "(ss)", INPUT_SOURCE_TYPE_XKB, tmp);
-                g_free (tmp);
+        for (i = 0; i < n && layouts[i][0]; ++i) {
+                gchar *id;
+
+                if (variants && variants[i] && variants[i][0])
+                        id = g_strdup_printf ("%s+%s", layouts[i], variants[i]);
+                else
+                        id = g_strdup (layouts[i]);
+
+                g_variant_builder_add (&builder, "(ss)", INPUT_SOURCE_TYPE_XKB, id);
+                g_free (id);
         }
-        g_settings_set_value (settings, KEY_INPUT_SOURCES, g_variant_builder_end (&builder));
-out:
+        g_settings_set_value (priv->input_sources_settings, KEY_INPUT_SOURCES, g_variant_builder_end (&builder));
+
         g_strfreev (layouts);
         g_strfreev (variants);
 }
 
+static void
+get_options_from_xkb_config (GsdKeyboardManager *manager)
+{
+        GsdKeyboardManagerPrivate *priv = manager->priv;
+        GVariant *v;
+        gchar **options = NULL;
+
+        v = g_dbus_proxy_get_cached_property (priv->localed, "X11Options");
+        if (v) {
+                const gchar *s = g_variant_get_string (v, NULL);
+                if (*s)
+                        options = g_strsplit (s, ",", -1);
+                g_variant_unref (v);
+        }
+
+        if (!options)
+                return;
+
+        g_settings_set_strv (priv->input_sources_settings, KEY_KEYBOARD_OPTIONS, (const gchar * const*) options);
+
+        g_strfreev (options);
+}
+
 static void
 convert_libgnomekbd_options (GSettings *settings)
 {
@@ -1456,15 +1451,23 @@ out:
 }
 
 static void
-maybe_create_input_sources (GsdKeyboardManager *manager)
+maybe_create_initial_settings (GsdKeyboardManager *manager)
 {
         GSettings *settings;
         GVariant *sources;
+        gchar **options;
 
         settings = manager->priv->input_sources_settings;
 
         if (g_getenv ("RUNNING_UNDER_GDM")) {
-                create_sources_from_current_xkb_config (settings);
+                GVariantBuilder builder;
+                /* clean the settings and get them from the "system" */
+                g_variant_builder_init (&builder, G_VARIANT_TYPE ("a(ss)"));
+                g_settings_set_value (settings, KEY_INPUT_SOURCES, g_variant_builder_end (&builder));
+                get_sources_from_xkb_config (manager);
+
+                g_settings_set_strv (settings, KEY_KEYBOARD_OPTIONS, NULL);
+                get_options_from_xkb_config (manager);
                 return;
         }
 
@@ -1473,12 +1476,38 @@ maybe_create_input_sources (GsdKeyboardManager *manager)
         /* if we still don't have anything do some educated guesses */
         sources = g_settings_get_value (settings, KEY_INPUT_SOURCES);
         if (g_variant_n_children (sources) < 1) {
-                create_sources_from_current_xkb_config (settings);
+                get_sources_from_xkb_config (manager);
 #ifdef HAVE_IBUS
                 add_ibus_sources_from_locale (settings);
 #endif
         }
         g_variant_unref (sources);
+
+        options = g_settings_get_strv (settings, KEY_KEYBOARD_OPTIONS);
+        if (g_strv_length (options) < 1)
+                get_options_from_xkb_config (manager);
+        g_strfreev (options);
+}
+
+static void
+localed_proxy_ready (GObject      *source,
+                     GAsyncResult *res,
+                     gpointer      data)
+{
+        GsdKeyboardManager *manager = data;
+        GError *error = NULL;
+
+        manager->priv->localed = g_dbus_proxy_new_finish (res, &error);
+
+        if (!manager->priv->localed) {
+                g_warning ("Failed to contact localed: %s", error->message);
+                g_error_free (error);
+                goto out;
+        }
+
+        maybe_create_initial_settings (manager);
+out:
+        apply_input_sources_settings (manager->priv->input_sources_settings, NULL, 0, manager);
 }
 
 static gboolean
@@ -1498,9 +1527,16 @@ start_keyboard_idle_cb (GsdKeyboardManager *manager)
         manager->priv->interface_settings = g_settings_new (GNOME_DESKTOP_INTERFACE_DIR);
         manager->priv->xkb_info = gnome_xkb_info_new ();
 
-        maybe_create_input_sources (manager);
+        g_dbus_proxy_new_for_bus (G_BUS_TYPE_SYSTEM,
+                                  G_DBUS_PROXY_FLAGS_NONE,
+                                  NULL,
+                                  "org.freedesktop.locale1",
+                                  "/org/freedesktop/locale1",
+                                  "org.freedesktop.locale1",
+                                  NULL,
+                                  localed_proxy_ready,
+                                  manager);
 
-        apply_input_sources_settings (manager->priv->input_sources_settings, NULL, 0, manager);
         /* apply current settings before we install the callback */
         g_debug ("Started the keyboard plugin, applying all settings");
         apply_all_settings (manager);
@@ -1549,6 +1585,7 @@ gsd_keyboard_manager_stop (GsdKeyboardManager *manager)
         g_clear_object (&p->input_sources_settings);
         g_clear_object (&p->interface_settings);
         g_clear_object (&p->xkb_info);
+        g_clear_object (&p->localed);
 
 #ifdef HAVE_IBUS
         clear_ibus (manager);
