@@ -53,7 +53,6 @@
 
 #include "shortcuts-list.h"
 #include "shell-key-grabber.h"
-#include "gsd-osd-window.h"
 #include "gsd-screenshot-utils.h"
 #include "gsd-input-helper.h"
 #include "gsd-enums.h"
@@ -150,7 +149,6 @@ struct GsdMediaKeysManagerPrivate
         GUdevClient     *udev_client;
 #endif /* HAVE_GUDEV */
 
-        GtkWidget       *dialog;
         GSettings       *settings;
         GHashTable      *custom_settings;
 
@@ -167,17 +165,17 @@ struct GsdMediaKeysManagerPrivate
         GDBusProxy      *power_screen_proxy;
         GDBusProxy      *power_keyboard_proxy;
 
-        /* shell grabber */
+        /* Shell stuff */
+        guint           name_owner_id;
+        GDBusProxy      *osd_proxy;
         ShellKeyGrabber *key_grabber;
+        GCancellable    *osd_cancellable;
         GCancellable    *grab_cancellable;
 
         /* systemd stuff */
         GDBusProxy      *logind_proxy;
         gint             inhibit_keys_fd;
 
-        GdkScreen       *screen;
-        int              opcode;
-
         GList           *media_players;
 
         GDBusNodeInfo   *introspection_data;
@@ -342,20 +340,6 @@ execute (GsdMediaKeysManager *manager,
         g_free (exec);
 }
 
-static void
-dialog_init (GsdMediaKeysManager *manager)
-{
-        if (manager->priv->dialog != NULL
-            && !gsd_osd_window_is_valid (GSD_OSD_WINDOW (manager->priv->dialog))) {
-                gtk_widget_destroy (manager->priv->dialog);
-                manager->priv->dialog = NULL;
-        }
-
-        if (manager->priv->dialog == NULL) {
-                manager->priv->dialog = gsd_osd_window_new ();
-        }
-}
-
 static char *
 get_key_string (GsdMediaKeysManager *manager,
 		MediaKey            *key)
@@ -375,17 +359,99 @@ get_key_string (GsdMediaKeysManager *manager,
 }
 
 static void
-ensure_grab_cancellable (GsdMediaKeysManager *manager)
+ensure_cancellable (GCancellable **cancellable)
 {
-        if (manager->priv->grab_cancellable == NULL) {
-                manager->priv->grab_cancellable = g_cancellable_new ();
-                g_object_add_weak_pointer (G_OBJECT (manager->priv->grab_cancellable),
-                                           (gpointer *)&manager->priv->grab_cancellable);
+        if (*cancellable == NULL) {
+                *cancellable = g_cancellable_new ();
+                g_object_add_weak_pointer (G_OBJECT (*cancellable),
+                                           (gpointer *)cancellable);
         } else {
-                g_object_ref (manager->priv->grab_cancellable);
+                g_object_ref (*cancellable);
         }
 }
 
+static void
+show_osd_complete (GObject      *source,
+                   GAsyncResult *result,
+                   gpointer      data)
+{
+        GsdMediaKeysManager *manager = data;
+        g_object_unref (manager->priv->osd_cancellable);
+}
+
+static void
+show_osd (GsdMediaKeysManager *manager,
+          const char          *icon,
+          const char          *label,
+          int                  level)
+{
+        GVariantBuilder builder;
+
+        if (manager->priv->osd_proxy == NULL)
+                return;
+
+        g_variant_builder_init (&builder, G_VARIANT_TYPE_TUPLE);
+        g_variant_builder_open (&builder, G_VARIANT_TYPE_VARDICT);
+        if (icon)
+                g_variant_builder_add (&builder, "{sv}",
+                                       "icon", g_variant_new_string (icon));
+        if (label)
+                g_variant_builder_add (&builder, "{sv}",
+                                       "label", g_variant_new_string (label));
+        if (level >= 0)
+                g_variant_builder_add (&builder, "{sv}",
+                                       "level", g_variant_new_int32 (level));
+        g_variant_builder_close (&builder);
+
+        ensure_cancellable (&manager->priv->osd_cancellable);
+        g_dbus_proxy_call (manager->priv->osd_proxy,
+                           "ShowOSD",
+                           g_variant_builder_end (&builder),
+                           G_DBUS_CALL_FLAGS_NO_AUTO_START,
+                           -1,
+                           manager->priv->osd_cancellable,
+                           show_osd_complete, manager);
+}
+
+static const char *
+get_icon_name_for_volume (gboolean is_mic,
+                          gboolean muted,
+                          int volume)
+{
+        static const char *icon_names[] = {
+                "audio-volume-muted-symbolic",
+                "audio-volume-low-symbolic",
+                "audio-volume-medium-symbolic",
+                "audio-volume-high-symbolic",
+                NULL
+        };
+        static const char *mic_icon_names[] = {
+                "microphone-sensitivity-muted-symbolic",
+                "microphone-sensitivity-low-symbolic",
+                "microphone-sensitivity-medium-symbolic",
+                "microphone-sensitivity-high-symbolic",
+                NULL
+        };
+        int n;
+
+        if (muted) {
+                n = 0;
+        } else {
+                /* select image */
+                n = 3 * volume / 100 + 1;
+                if (n < 1) {
+                        n = 1;
+                } else if (n > 3) {
+                        n = 3;
+                }
+        }
+
+	if (is_mic)
+		return mic_icon_names[n];
+	else
+		return icon_names[n];
+}
+
 static gboolean
 retry_grabs (gpointer data)
 {
@@ -448,7 +514,7 @@ grab_media_keys (GsdMediaKeysManager *manager)
                 g_free (tmp);
         }
 
-	ensure_grab_cancellable (manager);
+	ensure_cancellable (&manager->priv->grab_cancellable);
 	shell_key_grabber_call_grab_accelerators (manager->priv->key_grabber,
 	                                          g_variant_builder_end (&builder),
 	                                          manager->priv->grab_cancellable,
@@ -486,7 +552,7 @@ grab_media_key (MediaKey            *key,
 	data->manager = manager;
 	data->key = key;
 
-	ensure_grab_cancellable (manager);
+	ensure_cancellable (&manager->priv->grab_cancellable);
 	shell_key_grabber_call_grab_accelerator (manager->priv->key_grabber,
 	                                         tmp, key->modes,
 	                                         manager->priv->grab_cancellable,
@@ -514,7 +580,7 @@ ungrab_media_key (MediaKey            *key,
 	if (key->accel_id == 0)
 		return;
 
-	ensure_grab_cancellable (manager);
+	ensure_cancellable (&manager->priv->grab_cancellable);
 	shell_key_grabber_call_ungrab_accelerator (manager->priv->key_grabber,
 	                                           key->accel_id,
 	                                           manager->priv->grab_cancellable,
@@ -734,47 +800,6 @@ init_kbd (GsdMediaKeysManager *manager)
         gnome_settings_profile_end (NULL);
 }
 
-static void
-dialog_show (GsdMediaKeysManager *manager)
-{
-        int            orig_w;
-        int            orig_h;
-        int            screen_w;
-        int            screen_h;
-        int            x;
-        int            y;
-        GdkRectangle   geometry;
-        int            monitor;
-
-        gtk_window_set_screen (GTK_WINDOW (manager->priv->dialog),
-                               manager->priv->screen);
-
-        /*
-         * get the window size
-         * if the window hasn't been mapped, it doesn't necessarily
-         * know its true size, yet, so we need to jump through hoops
-         */
-        gtk_window_get_default_size (GTK_WINDOW (manager->priv->dialog), &orig_w, &orig_h);
-
-        monitor = gdk_screen_get_primary_monitor (manager->priv->screen);
-
-        gdk_screen_get_monitor_geometry (manager->priv->screen,
-                                         monitor,
-                                         &geometry);
-
-        screen_w = geometry.width;
-        screen_h = geometry.height;
-
-        x = ((screen_w - orig_w) / 2) + geometry.x;
-        y = geometry.y + (screen_h / 2) + (screen_h / 2 - orig_h) / 2;
-
-        gtk_window_move (GTK_WINDOW (manager->priv->dialog), x, y);
-
-        gtk_widget_show (manager->priv->dialog);
-
-        gdk_display_sync (gdk_screen_get_display (manager->priv->screen));
-}
-
 static void
 launch_app (GAppInfo *app_info,
 	    gint64    timestamp)
@@ -896,14 +921,8 @@ do_eject_action (GsdMediaKeysManager *manager)
                 }
         }
 
-        /* Show the dialogue */
-        dialog_init (manager);
-        gsd_osd_window_set_volume_label (GSD_OSD_WINDOW (manager->priv->dialog),
-                                         NULL);
-        gsd_osd_window_set_action_custom (GSD_OSD_WINDOW (manager->priv->dialog),
-                                          "media-eject-symbolic",
-                                          FALSE);
-        dialog_show (manager);
+        /* Show OSD */
+        show_osd (manager, "media-eject-symbolic", NULL, -1);
 
         /* Clean up the drive selection and exit if no suitable
          * drives are found */
@@ -966,13 +985,8 @@ do_execute_desktop_or_desktop (GsdMediaKeysManager *manager,
 static void
 do_touchpad_osd_action (GsdMediaKeysManager *manager, gboolean state)
 {
-        dialog_init (manager);
-        gsd_osd_window_set_volume_label (GSD_OSD_WINDOW (manager->priv->dialog),
-                                         NULL);
-        gsd_osd_window_set_action_custom (GSD_OSD_WINDOW (manager->priv->dialog),
-                                          state ? "input-touchpad-symbolic" : "touchpad-disabled-symbolic",
-                                          FALSE);
-        dialog_show (manager);
+        show_osd (manager, state ? "input-touchpad-symbolic"
+                                 : "touchpad-disabled-symbolic", NULL, -1);
 }
 
 static void
@@ -1024,7 +1038,7 @@ update_dialog (GsdMediaKeysManager *manager,
 {
         GvcMixerUIDevice *device;
         const GvcMixerStreamPort *port;
-        GsdOsdWindowAction action;
+        const char *icon;
 
         if (!muted) {
                 vol = (int) (100 * (double) vol / PA_VOLUME_NORM);
@@ -1033,23 +1047,17 @@ update_dialog (GsdMediaKeysManager *manager,
                 vol = 0.0;
         }
 
-        action = GVC_IS_MIXER_SINK (stream) ? GSD_OSD_WINDOW_ACTION_VOLUME : GSD_OSD_WINDOW_ACTION_MIC_VOLUME;
-
-        dialog_init (manager);
-        gsd_osd_window_set_volume_muted (GSD_OSD_WINDOW (manager->priv->dialog), muted);
-        gsd_osd_window_set_volume_level (GSD_OSD_WINDOW (manager->priv->dialog), vol);
-        gsd_osd_window_set_action (GSD_OSD_WINDOW (manager->priv->dialog), action);
-
+        icon = get_icon_name_for_volume (!GVC_IS_MIXER_SINK (stream), muted, vol);
         port = gvc_mixer_stream_get_port (stream);
         if (g_strcmp0 (gvc_mixer_stream_get_form_factor (stream), "internal") != 0 ||
             g_strcmp0 (port->port, "analog-output-speaker") != 0) {
                 device = gvc_mixer_control_lookup_device_from_stream (manager->priv->volume, stream);
-                gsd_osd_window_set_volume_label (GSD_OSD_WINDOW (manager->priv->dialog),
-                                                 gvc_mixer_ui_device_get_description (device));
+                show_osd (manager, icon,
+                          gvc_mixer_ui_device_get_description (device), vol);
+        } else {
+                show_osd (manager, icon, NULL, vol);
         }
 
-        dialog_show (manager);
-
         if (quiet == FALSE && sound_changed != FALSE && muted == FALSE) {
                 ca_context_change_device (manager->priv->ca,
                                           gvc_mixer_stream_get_name (stream));
@@ -1505,13 +1513,7 @@ gsd_media_player_key_pressed (GsdMediaKeysManager *manager,
 
         if (!have_listeners) {
                 /* Popup a dialog with an (/) icon */
-                dialog_init (manager);
-                gsd_osd_window_set_volume_label (GSD_OSD_WINDOW (manager->priv->dialog),
-                                                 NULL);
-                gsd_osd_window_set_action_custom (GSD_OSD_WINDOW (manager->priv->dialog),
-                                                  "action-unavailable-symbolic",
-                                                  FALSE);
-                dialog_show (manager);
+                show_osd (manager, "action-unavailable-symbolic", NULL, -1);
                 return TRUE;
         }
 
@@ -1834,15 +1836,7 @@ update_screen_cb (GObject             *source_object,
 
         /* update the dialog with the new value */
         g_variant_get (new_percentage, "(u)", &percentage);
-        dialog_init (manager);
-        gsd_osd_window_set_volume_label (GSD_OSD_WINDOW (manager->priv->dialog),
-                                         NULL);
-        gsd_osd_window_set_action_custom (GSD_OSD_WINDOW (manager->priv->dialog),
-                                          "display-brightness-symbolic",
-                                          TRUE);
-        gsd_osd_window_set_volume_level (GSD_OSD_WINDOW (manager->priv->dialog),
-                                                percentage);
-        dialog_show (manager);
+        show_osd (manager, "display-brightness-symbolic", NULL, percentage);
         g_variant_unref (new_percentage);
 }
 
@@ -1888,15 +1882,7 @@ update_keyboard_cb (GObject             *source_object,
 
         /* update the dialog with the new value */
         g_variant_get (new_percentage, "(u)", &percentage);
-        dialog_init (manager);
-        gsd_osd_window_set_volume_label (GSD_OSD_WINDOW (manager->priv->dialog),
-                                         NULL);
-        gsd_osd_window_set_action_custom (GSD_OSD_WINDOW (manager->priv->dialog),
-                                          "keyboard-brightness-symbolic",
-                                          TRUE);
-        gsd_osd_window_set_volume_level (GSD_OSD_WINDOW (manager->priv->dialog),
-                                                percentage);
-        dialog_show (manager);
+        show_osd (manager, "keyboard-brightness-symbolic", NULL, percentage);
         g_variant_unref (new_percentage);
 }
 
@@ -1941,8 +1927,7 @@ static void
 do_battery_action (GsdMediaKeysManager *manager)
 {
         GVariant *icon_var, *percentage;
-        GIcon *icon;
-        gboolean show_percentage = FALSE;
+        char *label = NULL;
 
         if (manager->priv->power_proxy == NULL)
                 return;
@@ -1950,23 +1935,12 @@ do_battery_action (GsdMediaKeysManager *manager)
         icon_var = g_dbus_proxy_get_cached_property (manager->priv->power_proxy, "Icon");
         percentage = g_dbus_proxy_get_cached_property (manager->priv->power_proxy, "Percentage");
 
-        dialog_init (manager);
-        if (g_variant_get_double (percentage) >= 0.0) {
-                char *str;
+        if (g_variant_get_double (percentage) >= 0.0)
+                label = g_strdup_printf ("%d %%", (int) g_variant_get_double (percentage));
 
-                show_percentage = TRUE;
-                str = g_strdup_printf ("%d %%", (int) g_variant_get_double (percentage));
-                gsd_osd_window_set_volume_level (GSD_OSD_WINDOW (manager->priv->dialog),
-                                                 g_variant_get_double (percentage));
-                gsd_osd_window_set_volume_label (GSD_OSD_WINDOW (manager->priv->dialog),
-                                                 str);
-                g_free (str);
-        }
-        icon = g_icon_new_for_string (g_variant_get_string (icon_var, NULL), NULL);
-        gsd_osd_window_set_action_custom_gicon (GSD_OSD_WINDOW (manager->priv->dialog),
-                                                icon,
-                                                show_percentage);
-        dialog_show (manager);
+        show_osd (manager, g_variant_get_string (icon_var, NULL),
+                  label, g_variant_get_double (percentage));
+        g_free (label);
 }
 
 static void
@@ -2132,24 +2106,6 @@ do_action (GsdMediaKeysManager *manager,
         return FALSE;
 }
 
-static void
-on_shell_restart (GObject             *object,
-                  GParamSpec          *pspec,
-                  GsdMediaKeysManager *manager)
-{
-        char *owner;
-
-        if (manager->priv->keys == NULL)
-                return;
-
-        g_object_get (object, "g-name-owner", &owner, NULL);
-        if (!owner)
-                return;
-        g_free (owner);
-
-        grab_media_keys (manager);
-}
-
 static void
 on_accelerator_activated (ShellKeyGrabber     *grabber,
                           guint                accel_id,
@@ -2230,6 +2186,18 @@ initialize_volume_handler (GsdMediaKeysManager *manager)
         gnome_settings_profile_end ("gvc_mixer_control_new");
 }
 
+static void
+on_osd_proxy_ready (GObject      *source,
+                    GAsyncResult *result,
+                    gpointer      data)
+{
+        GsdMediaKeysManager *manager = data;
+
+        manager->priv->osd_proxy =
+                g_dbus_proxy_new_for_bus_finish (result, NULL);
+        g_object_unref (manager->priv->osd_cancellable);
+}
+
 static void
 on_key_grabber_ready (GObject      *source,
                       GAsyncResult *result,
@@ -2245,14 +2213,47 @@ on_key_grabber_ready (GObject      *source,
         if (!manager->priv->key_grabber)
                 return;
 
-        g_signal_connect (manager->priv->key_grabber, "notify::g-name-owner",
-                          G_CALLBACK (on_shell_restart), manager);
         g_signal_connect (manager->priv->key_grabber, "accelerator-activated",
                           G_CALLBACK (on_accelerator_activated), manager);
 
         init_kbd (manager);
 }
 
+static void
+on_shell_appeared (GDBusConnection   *connection,
+                   const char        *name,
+                   const char        *name_owner,
+                   gpointer           user_data)
+{
+        GsdMediaKeysManager *manager = user_data;
+        
+        shell_key_grabber_proxy_new_for_bus (G_BUS_TYPE_SESSION,
+                                             0,
+                                             name_owner,
+                                             SHELL_DBUS_PATH,
+                                             manager->priv->grab_cancellable,
+                                             on_key_grabber_ready, manager);
+        
+        ensure_cancellable (&manager->priv->osd_cancellable);
+        g_dbus_proxy_new_for_bus (G_BUS_TYPE_SESSION,
+                                  0, NULL,
+                                  name_owner,
+                                  SHELL_DBUS_PATH,
+                                  SHELL_DBUS_NAME,
+                                  manager->priv->osd_cancellable,
+                                  on_osd_proxy_ready, manager);
+}
+
+static void
+on_shell_vanished (GDBusConnection  *connection,
+                   const char       *name,
+                   gpointer          user_data)
+{
+        GsdMediaKeysManager *manager = user_data;
+        g_clear_object (&manager->priv->key_grabber);
+        g_clear_object (&manager->priv->osd_proxy);
+}
+
 static gboolean
 start_media_keys_idle_cb (GsdMediaKeysManager *manager)
 {
@@ -2303,15 +2304,14 @@ start_media_keys_idle_cb (GsdMediaKeysManager *manager)
 	}
 	manager->priv->icon_theme = g_settings_get_string (manager->priv->interface_settings, "icon-theme");
 
-        manager->priv->screen = gdk_display_get_screen (gdk_display_get_default (), 0);
+        ensure_cancellable (&manager->priv->grab_cancellable);
 
-        ensure_grab_cancellable (manager);
-        shell_key_grabber_proxy_new_for_bus (G_BUS_TYPE_SESSION,
-                                             G_DBUS_PROXY_FLAGS_DO_NOT_AUTO_START,
-                                             SHELL_DBUS_NAME,
-                                             SHELL_DBUS_PATH,
-                                             manager->priv->grab_cancellable,
-                                             on_key_grabber_ready, manager);
+        manager->priv->name_owner_id =
+                g_bus_watch_name (G_BUS_TYPE_SESSION,
+                                  SHELL_DBUS_NAME, 0,
+                                  on_shell_appeared,
+                                  on_shell_vanished,
+                                  manager, NULL);
 
         gnome_settings_profile_end (NULL);
 
@@ -2328,11 +2328,6 @@ gsd_media_keys_manager_start (GsdMediaKeysManager *manager,
 
         gnome_settings_profile_start (NULL);
 
-        if (supports_xinput2_devices (&manager->priv->opcode) == FALSE) {
-                g_debug ("No Xinput2 support, disabling plugin");
-                return TRUE;
-        }
-
 #ifdef HAVE_GUDEV
         manager->priv->streams = g_hash_table_new (g_direct_hash, g_direct_equal);
         manager->priv->udev_client = g_udev_client_new (subsystems);
@@ -2380,6 +2375,11 @@ gsd_media_keys_manager_stop (GsdMediaKeysManager *manager)
         g_clear_object (&priv->power_screen_proxy);
         g_clear_object (&priv->power_keyboard_proxy);
 
+        if (manager->priv->name_owner_id) {
+                g_bus_unwatch_name (manager->priv->name_owner_id);
+                manager->priv->name_owner_id = 0;
+        }
+
         if (priv->cancellable != NULL) {
                 g_cancellable_cancel (priv->cancellable);
                 g_clear_object (&priv->cancellable);
@@ -2404,10 +2404,14 @@ gsd_media_keys_manager_stop (GsdMediaKeysManager *manager)
                 g_clear_object (&priv->grab_cancellable);
         }
 
+        if (priv->osd_cancellable != NULL) {
+                g_cancellable_cancel (priv->osd_cancellable);
+                g_clear_object (&priv->osd_cancellable);
+        }
+
         g_clear_object (&priv->sink);
         g_clear_object (&priv->source);
         g_clear_object (&priv->volume);
-        g_clear_object (&priv->dialog);
 
         if (priv->media_players != NULL) {
                 g_list_free_full (priv->media_players, (GDestroyNotify) free_media_player);
