@@ -69,7 +69,6 @@ struct GsdUpdatesManagerPrivate
 
 static void gsd_updates_manager_class_init (GsdUpdatesManagerClass *klass);
 static void gsd_updates_manager_init (GsdUpdatesManager *updates_manager);
-static void gsd_updates_manager_finalize (GObject *object);
 
 G_DEFINE_TYPE (GsdUpdatesManager, gsd_updates_manager, G_TYPE_OBJECT)
 
@@ -284,6 +283,10 @@ get_distro_upgrades_finished_cb (GObject *object,
         /* get the results */
         results = pk_client_generic_finish (PK_CLIENT(client), res, &error);
         if (results == NULL) {
+                if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED)) {
+                        g_error_free (error);
+                        return;
+                }
                 if (error->domain != PK_CLIENT_ERROR ||
                     error->code != PK_CLIENT_ERROR_NOT_SUPPORTED) {
                         g_warning ("failed to get upgrades: %s",
@@ -394,10 +397,14 @@ refresh_cache_finished_cb (GObject *object, GAsyncResult *res, GsdUpdatesManager
         /* get the results */
         results = pk_client_generic_finish (PK_CLIENT(client), res, &error);
         if (results == NULL) {
+                if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED)) {
+                        g_error_free (error);
+                        return;
+                }
                 g_warning ("failed to refresh the cache: %s",
                            error->message);
                 g_error_free (error);
-                goto out;
+                return;
         }
 
         /* check error code */
@@ -406,9 +413,8 @@ refresh_cache_finished_cb (GObject *object, GAsyncResult *res, GsdUpdatesManager
                 g_warning ("failed to refresh the cache: %s, %s",
                            pk_error_enum_to_string (pk_error_get_code (error_code)),
                            pk_error_get_details (error_code));
-                goto out;
         }
-out:
+
         if (error_code != NULL)
                 g_object_unref (error_code);
         if (results != NULL)
@@ -643,11 +649,15 @@ package_download_finished_cb (GObject *object,
         /* get the results */
         results = pk_client_generic_finish (PK_CLIENT(client), res, &error);
         if (results == NULL) {
+                if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED)) {
+                        g_error_free (error);
+                        return;
+                }
                 g_warning ("failed to download: %s",
                            error->message);
                 g_error_free (error);
                 notify_failed_get_updates_maybe (manager);
-                goto out;
+                return;
         }
 
         /* check error code */
@@ -727,6 +737,10 @@ get_updates_finished_cb (GObject *object,
         /* get the results */
         results = pk_client_generic_finish (PK_CLIENT(client), res, &error);
         if (results == NULL) {
+                if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED)) {
+                        g_error_free (error);
+                        return;
+                }
                 g_warning ("failed to get updates: %s",
                            error->message);
                 g_error_free (error);
@@ -900,7 +914,8 @@ set_proxy_cb (GObject *object, GAsyncResult *res, gpointer user_data)
         /* get the result */
         ret = pk_control_set_proxy_finish (control, res, &error);
         if (!ret) {
-                g_warning ("failed to set proxies: %s", error->message);
+                if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
+                        g_warning ("failed to set proxies: %s", error->message);
                 g_error_free (error);
         }
 }
@@ -1359,7 +1374,10 @@ gsd_updates_manager_stop (GsdUpdatesManager *manager)
         g_clear_object (&manager->priv->firmware);
         g_clear_object (&manager->priv->proxy_session);
         g_clear_object (&manager->priv->volume_monitor);
-        g_clear_object (&manager->priv->cancellable);
+        if (manager->priv->cancellable) {
+                g_cancellable_cancel (manager->priv->cancellable);
+                g_clear_object (&manager->priv->cancellable);
+        }
         if (manager->priv->update_viewer_watcher_id != 0) {
                 g_bus_unwatch_name (manager->priv->update_viewer_watcher_id);
                 manager->priv->update_viewer_watcher_id = 0;
@@ -1410,7 +1428,6 @@ gsd_updates_manager_class_init (GsdUpdatesManagerClass *klass)
 
         object_class->constructor = gsd_updates_manager_constructor;
         object_class->dispose = gsd_updates_manager_dispose;
-        object_class->finalize = gsd_updates_manager_finalize;
 
         g_type_class_add_private (klass, sizeof (GsdUpdatesManagerPrivate));
 }
@@ -1421,21 +1438,6 @@ gsd_updates_manager_init (GsdUpdatesManager *manager)
         manager->priv = GSD_UPDATES_MANAGER_GET_PRIVATE (manager);
 }
 
-static void
-gsd_updates_manager_finalize (GObject *object)
-{
-        GsdUpdatesManager *updates_manager;
-
-        g_return_if_fail (object != NULL);
-        g_return_if_fail (GSD_IS_UPDATES_MANAGER (object));
-
-        updates_manager = GSD_UPDATES_MANAGER (object);
-
-        g_return_if_fail (updates_manager->priv);
-
-        G_OBJECT_CLASS (gsd_updates_manager_parent_class)->finalize (object);
-}
-
 GsdUpdatesManager *
 gsd_updates_manager_new (void)
 {
